﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.MainForm.WindowsForms;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Problems.Instances;
using HeuristicLab.Problems.DataAnalysis;
using HeuristicLab.Problems.DataAnalysis.Symbolic;
using HeuristicLab.Problems.DataAnalysis.Symbolic.Regression;
using HeuristicLab.Optimizer;
using HeuristicLab.MainForm;

namespace HeuristicLab.VariableInteractionNetworks
{
    public partial class CreateTargetVariationExperimentDialog : Form
    {
        public Experiment Experiment { get; private set; }
        private int repetitions;
        
        public CreateTargetVariationExperimentDialog() : this(null) { }
        public CreateTargetVariationExperimentDialog(IAlgorithm alg)
        {
            InitializeComponent();
            repetitions = (int)repetitionsNumericUpDown.Value;

            Experiment = null;
            okButton.Enabled = alg != null;
        }

        private void repetitionsNumericUpDown_Validated(object sender, EventArgs e)
        {
            if (repetitionsNumericUpDown.Text == string.Empty)
                repetitionsNumericUpDown.Text = repetitionsNumericUpDown.Value.ToString();
            repetitions = (int)repetitionsNumericUpDown.Value;
        }

        public Experiment CreateVariableCombinations(IAlgorithm algorithm)
        {
            var experiment = new Experiment("Target Variation Experiment");

            if (algorithm == null)
                throw new InvalidOperationException("Null algorithm!");

            var regressionProblem = algorithm.Problem as IRegressionProblem;
            if (regressionProblem == null)
                throw new InvalidOperationException("Null regression problem!!");

            var problemData = regressionProblem.ProblemData;
            var variables = problemData.InputVariables.ToList();
            int checksNo = 0;

            for (int i = 0; i < variables.Count; i++)
            {
                if (!problemData.InputVariables.ItemChecked(problemData.InputVariables[i]))
                    continue;

                var alg = (IAlgorithm)algorithm.Clone();
                var pd = ((IRegressionProblem)alg.Problem).ProblemData;
                checksNo++;

                // set target to false
                pd.InputVariables.SetItemCheckedState(pd.InputVariables[i], false);
                    
                pd.TargetVariable = pd.InputVariables[i].Value;
                    
                alg.Name += "{Target variable=" + pd.TargetVariable + "}";

                var batchrun = new BatchRun("Batchrun" + checksNo)
                {
                    Repetitions = repetitions,
                    Optimizer = alg
                };

                experiment.Optimizers.Add(batchrun);
            }
            return experiment;
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            IContentView activeView = MainFormManager.MainForm.ActiveView as IContentView;
            if ((activeView != null) && (activeView.Content != null) && (activeView.Content is IOptimizer) && !activeView.Locked)
            {
                Experiment = CreateVariableCombinations((IAlgorithm)activeView.Content);
                DialogResult = System.Windows.Forms.DialogResult.OK;
                Close();
            }
        }
    }
}